//
// Copyright 2009-2011 Facebook
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#import "TTTableImageItemCell.h"

// UI
#import "TTImageView.h"
#import "TTTableImageItem.h"
#import "TTTableRightImageItem.h"
#import "UIViewAdditions.h"

// Style
#import "TTGlobalStyle.h"
#import "TTDefaultStyleSheet.h"
#import "TTImageStyle.h"

// Network
#import "TTURLCache.h"

// Core
#import "TTCorePreprocessorMacros.h"

static const CGFloat kKeySpacing = 12.0f;
static const CGFloat kDefaultImageSize = 50.0f;


///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
@implementation TTTableImageItemCell

@synthesize imageView2 = _imageView2;


///////////////////////////////////////////////////////////////////////////////////////////////////
- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString*)identifier {
	self = [super initWithStyle:style reuseIdentifier:identifier];
  if (self) {
    _imageView2 = [[TTImageView alloc] init];
    [self.contentView addSubview:_imageView2];
  }

  return self;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (void)dealloc {
  TT_RELEASE_SAFELY(_imageView2);

  [super dealloc];
}


///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
#pragma mark -
#pragma mark Private helpers


///////////////////////////////////////////////////////////////////////////////////////////////////
+ (UIFont*)fontForImageItem:(id)imageItem {
  if ([imageItem isKindOfClass:[TTTableRightImageItem class]]) {
    return TTSTYLEVAR(tableSmallFont);

  } else {
    return TTSTYLEVAR(tableFont);
  }
}


///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
#pragma mark -
#pragma mark TTTableViewCell class public


///////////////////////////////////////////////////////////////////////////////////////////////////
+ (CGFloat)tableView:(UITableView*)tableView rowHeightForObject:(id)object {
  TTTableImageItem* imageItem = object;

  UIImage* image = imageItem.imageURL
  ? [[TTURLCache sharedCache] imageForURL:imageItem.imageURL] : nil;
  if (!image) {
    image = imageItem.defaultImage;
  }

  CGFloat imageHeight, imageWidth;
  TTImageStyle* style = [imageItem.imageStyle firstStyleOfClass:[TTImageStyle class]];
  if (style && !CGSizeEqualToSize(style.size, CGSizeZero)) {
    imageWidth = style.size.width + kKeySpacing;
    imageHeight = style.size.height;

  } else {
    imageWidth = image
    ? image.size.width + kKeySpacing
    : (imageItem.imageURL ? kDefaultImageSize + kKeySpacing : 0);
    imageHeight = image
    ? image.size.height
    : (imageItem.imageURL ? kDefaultImageSize : 0);
  }

  CGFloat maxWidth = tableView.width - (imageWidth + kTableCellHPadding*2 + kTableCellMargin*2);

  CGSize textSize = [imageItem.text sizeWithFont:[self fontForImageItem:imageItem]
                               constrainedToSize:CGSizeMake(maxWidth, CGFLOAT_MAX)
                                   lineBreakMode:UILineBreakModeTailTruncation];

  CGFloat contentHeight = textSize.height > imageHeight ? textSize.height : imageHeight;
  return contentHeight + kTableCellVPadding*2;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
#pragma mark -
#pragma mark UIView


///////////////////////////////////////////////////////////////////////////////////////////////////
- (void)layoutSubviews {
  [super layoutSubviews];

  TTTableImageItem* item = self.object;
  UIImage* image = item.imageURL ? [[TTURLCache sharedCache] imageForURL:item.imageURL] : nil;
  if (!image) {
    image = item.defaultImage;
  }

  if ([_item isKindOfClass:[TTTableRightImageItem class]]) {
    CGFloat imageWidth = image
    ? image.size.width
    : (item.imageURL ? kDefaultImageSize : 0);
    CGFloat imageHeight = image
    ? image.size.height
    : (item.imageURL ? kDefaultImageSize : 0);

    if (_imageView2.urlPath || image) {
      CGFloat innerWidth = self.contentView.width - (kTableCellHPadding*2
                                                     + imageWidth + kKeySpacing);
      CGFloat innerHeight = self.contentView.height - kTableCellVPadding*2;
      self.textLabel.frame = CGRectMake(kTableCellHPadding, kTableCellVPadding,
                                        innerWidth, innerHeight);

      _imageView2.frame = CGRectMake(self.textLabel.right + kKeySpacing,
                                     floor(self.height/2 - imageHeight/2),
                                     imageWidth, imageHeight);

    } else {
      self.textLabel.frame = CGRectInset(self.contentView.bounds,
                                         kTableCellHPadding, kTableCellVPadding);
      _imageView2.frame = CGRectZero;
    }

  } else {
    if (_imageView2.urlPath || image) {
      CGFloat iconWidth = image
      ? image.size.width
      : (item.imageURL ? kDefaultImageSize : 0);
      CGFloat iconHeight = image
      ? image.size.height
      : (item.imageURL ? kDefaultImageSize : 0);

      TTImageStyle* style = [item.imageStyle firstStyleOfClass:[TTImageStyle class]];
      if (style) {
        _imageView2.contentMode = style.contentMode;
        _imageView2.clipsToBounds = YES;
        _imageView2.backgroundColor = [UIColor clearColor];
        if (style.size.width) {
          iconWidth = style.size.width;
        }
        if (style.size.height) {
          iconHeight = style.size.height;
        }
      }

      _imageView2.frame = CGRectMake(kTableCellHPadding, floor(self.height/2 - iconHeight/2),
                                     iconWidth, iconHeight);

      CGFloat innerWidth = self.contentView.width - (kTableCellHPadding*2
                                                     + iconWidth + kKeySpacing);
      CGFloat innerHeight = self.contentView.height - kTableCellVPadding*2;
      self.textLabel.frame = CGRectMake(kTableCellHPadding + iconWidth + kKeySpacing,
                                        kTableCellVPadding,
                                        innerWidth, innerHeight);

    } else {
      self.textLabel.frame = CGRectInset(self.contentView.bounds,
                                         kTableCellHPadding, kTableCellVPadding);
      _imageView2.frame = CGRectZero;
    }
  }
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (void)didMoveToSuperview {
  [super didMoveToSuperview];

  if (self.superview) {
    _imageView2.backgroundColor = self.backgroundColor;
  }
}


///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
#pragma mark -
#pragma mark TTTableViewCell


///////////////////////////////////////////////////////////////////////////////////////////////////
- (void)setObject:(id)object {
  if (_item != object) {
    [super setObject:object];

    TTTableImageItem* item = object;
    _imageView2.style = item.imageStyle;
    _imageView2.defaultImage = item.defaultImage;
    _imageView2.urlPath = item.imageURL;

    self.textLabel.font = [[self class] fontForImageItem:item];

    if ([_item isKindOfClass:[TTTableRightImageItem class]]) {
      self.textLabel.textAlignment = UITextAlignmentCenter;
      self.accessoryType = UITableViewCellAccessoryNone;

    } else {
      self.textLabel.textAlignment = UITextAlignmentLeft;
    }
  }
}


@end
